package main;

use strict;
use utf8;
use File::Basename;
use File::Find;
use Getopt::Long qw(:config no_ignore_case);

use constant OK => 0;
use constant WARNING => 1;
use constant CRITICAL => 2;
use constant UNKNOWN => 3;

use vars qw (%commandline);
my @cfgfiles = ();
my $needs_restart = 0;

my $plugin_revision = '$Revision: 1.0 $ ';
my $progname = basename($0);

sub print_version {
  printf "%s v#PACKAGE_VERSION#\n", basename($0);
}

sub print_help {
  print <<EOTXT;
This Nagios Plugin comes with absolutely NO WARRANTY. You may use
it on your own risk!
Copyright by ConSol Software GmbH, Gerhard Lausser.

This plugin looks for patterns in logfiles, even in those who were rotated
since the last run of this plugin.

Usage: check_logfiles [-t timeout] -f <configfile>

The configfile looks like this:

\$seekfilesdir = '/opt/nagios/var/tmp';
# where the state information will be saved.

\$protocolsdir = '/opt/nagios/var/tmp';
# where protocols with found patterns will be stored.

\$scriptpath = '/opt/nagios/var/tmp';
# where scripts will be searched for.

\$MACROS = \{ CL_DISK01 => "/dev/dsk/c0d1", CL_DISK02 => "/dev/dsk/c0d2" \};

\@searches = (
  {
    tag => 'temperature',
    logfile => '/var/adm/syslog/syslog.log',
    rotation => 'bmwhpux',
    criticalpatterns => ['OVERTEMP_EMERG', 'Power supply failed'],
    warningpatterns => ['OVERTEMP_CRIT', 'Corrected ECC Error'],
    options => 'script,protocol,nocount',
    script => 'sendnsca_cmd'
  },
  {
    tag => 'scsi',
    logfile => '/var/adm/messages',
    rotation => 'solaris',
    criticalpatterns => 'Sense Key: Not Ready',
    criticalexceptions => 'Sense Key: Not Ready /dev/testdisk',
    options => 'noprotocol'
  },
  {
    tag => 'logins',
    logfile => '/var/adm/messages',
    rotation => 'solaris',
    criticalpatterns => ['illegal key', 'read error.*\$CL_DISK01\$'],
    criticalthreshold => 4
    warningpatterns => ['read error.*\$CL_DISK02\$'],
  }
);

EOTXT
}

sub print_usage {
  print <<EOTXT;
Usage: check_logfiles [-t timeout] -f <configfile> [--searches=tag1,tag2,...]
       check_logfiles [-t timeout] --logfile=<logfile> --tag=<tag> --rotation=<rotation>
                      --criticalpattern=<regexp> --warningpattern=<regexp>
EOTXT
}

%commandline = ();
my @params = (
    "config|f=s",
    "configdir|F=s",
    "timeout|t=i",
    "version|V",
    "help|h",
    "debug|d",
    "verbose|v",
    "tag=s",
    "logfile=s",
    "rotation=s",
    "criticalpattern=s",
    "criticalexception=s",
    "warningpattern=s",
    "warningexception=s",
    "okpattern=s",
    "noprotocol",
    "syslogserver",
    "syslogclient=s",
    "sticky:s",
    "noperfdata",
    "searches=s",
    "selectedsearches=s",
    "template=s",
    "environment|e=s%"
);
if (! GetOptions(\%commandline, @params)) {
  print_help();
  exit $ERRORS{UNKNOWN};
} 

if (exists $commandline{version}) {
  print_version();
  exit UNKNOWN;
}

if (exists $commandline{help}) {
  print_help();
  exit UNKNOWN;
}

if (! (exists $commandline{config} || exists $commandline{configdir}) &&
     ! exists $commandline{logfile}) {
  print_usage();
  exit UNKNOWN;
}

if (exists $commandline{environment}) {
  # if the desired environment variable values are different from
  # the environment of this running script, then a restart is necessary.
  # because setting $ENV does _not_ change the environment of the running script.
  foreach (keys %{$commandline{environment}}) {
    if ((! $ENV{$_}) || ($ENV{$_} ne $commandline{environment}->{$_})) {
      $needs_restart = 1;
      $ENV{$_} = $commandline{environment}->{$_};
    }
  }
}
if ($needs_restart) {
  my @newargv = ();
  foreach my $option (keys %commandline) {
    if (grep { /^$option/ && /=/ } @params) {
      if (ref ($commandline{$option}) eq "HASH") {
        foreach (keys %{$commandline{$option}}) {
          push(@newargv, sprintf "--%s", $option);
          push(@newargv, sprintf "%s=%s", $_, $commandline{$option}->{$_});
        }
      } else {
        push(@newargv, sprintf "--%s", $option);
        push(@newargv, sprintf "%s", $commandline{$option});
      }
    } else {
      push(@newargv, sprintf "--%s", $option);
    }
  }
  exec $0, @newargv;
  # this makes sure that even a SHLIB or LD_LIBRARY_PATH are set correctly
  # when the perl interpreter starts. Setting them during runtime does not
  # help loading e.g. libclntsh.so
  exit;
}

if (exists $commandline{configdir}) {
  sub eachFile {
    my $filename = $_;
    my $fullpath = $File::Find::name;
    #remember that File::Find changes your CWD, 
    #so you can call open with just $_
    if ((-f $filename) && ($filename =~ /\.(cfg|conf)$/)) { 
      push(@cfgfiles, $fullpath);
    }
  }
  find (\&eachFile, $commandline{configdir});
  @cfgfiles = sort { $a cmp $b } @cfgfiles;
}
if (exists $commandline{config}) {
  # -f is always first
  unshift(@cfgfiles, $commandline{config});
}
if (scalar(@cfgfiles) == 1) {
  $commandline{config} = $cfgfiles[0];
} elsif (scalar(@cfgfiles) > 1) {
  $commandline{config} = \@cfgfiles;
}
if (exists $commandline{searches}) {
  $commandline{selectedsearches} = $commandline{searches};
}
if (! exists $commandline{selectedsearches}) {
  $commandline{selectedsearches} = "";
}
if (my $cl = Nagios::CheckLogfiles->new({
      cfgfile => $commandline{config} ? $commandline{config} : undef,
      searches => [ map { foreach my $key (keys %{$_}) { 
      	  delete $_->{$key} unless $_->{$key}}; $_; } ({
          tag => 
              $commandline{tag} ? $commandline{tag} : undef,
          logfile => 
              $commandline{logfile} ? $commandline{logfile} : undef,
          rotation => 
              $commandline{rotation} ? $commandline{rotation} : undef,
          criticalpatterns =>
              $commandline{criticalpattern} ?
                  $commandline{criticalpattern} : undef,
          criticalexceptions =>
              $commandline{criticalexception} ?
                  $commandline{criticalexception} : undef,
          warningpatterns =>
              $commandline{warningpattern} ?
                  $commandline{warningpattern} : undef,
          warningexceptions =>
              $commandline{warningexception} ?
                  $commandline{warningexception} : undef,
          okpatterns =>
              $commandline{okpattern} ?
                  $commandline{okpattern} : undef,
          options => join(',', grep { $_ }
              $commandline{noprotocol} ? "noprotocol" : undef,
              $commandline{noperfdata} ? "noperfdata" : undef,
              $commandline{syslogserver} ? "syslogserver" : undef,
              $commandline{syslogclient} ? "syslogclient=".$commandline{syslogclient} : undef,
              defined $commandline{sticky} ? "sticky".($commandline{sticky} ? "=".$commandline{sticky} : "") : undef )
      })],
      selectedsearches => [split(/,/, $commandline{selectedsearches})],
      dynamictag => $commandline{tag} ? $commandline{tag} : undef
  })) {
  $cl->{verbose} = $commandline{verbose} ? 1 : 0;
  $cl->{timeout} = $commandline{timeout} ? $commandline{timeout} : 60;
  $cl->run();
  printf "%s%s\n", $cl->{exitmessage}, $cl->{perfdata} ? "|".$cl->{perfdata} : "";
  exit $cl->{exitcode};
} else {
  printf "%s\n", $Nagios::CheckLogfiles::ExitMsg;
  exit $Nagios::CheckLogfiles::ExitCode;
}

